package com.isu.gaswellwatch.vo.command.scss;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import jakarta.validation.constraints.NotNull;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;

/**
 * 控制定值数据寄存器地址
 *
 * @author <a href="mailto:scwsl@foxmail.com>王仕龙</a>
 * 2024/11/28 0:15
 */
@Getter
@Setter
@SuperBuilder
public class ControlMode extends Command {

    @Serial
    private static final long serialVersionUID = -3782424709053622334L;

    /**
     * 控制模式
     */
    private int ctlModel;
    /**
     * 套压最小值
     */
    @NotNull(message = "套压最小值不能为空")
    private BigDecimal minPressure;
    /**
     * 套压最大值
     */
    @NotNull(message = "套压最大值不能为空")
    private BigDecimal maxPressure;
    /**
     * 套压最小电压
     */
    @NotNull(message = "套压最小电压不能为空")
    private BigDecimal pressureMinVoltage;
    /**
     * 套压最大电压
     */
    @NotNull(message = "套压最大电压不能为空")
    private BigDecimal pressureMaxVoltage;
    /**
     * 油压最小值
     */
    @NotNull(message = "油压最小值不能为空")
    private BigDecimal oilMin;
    /**
     * 油压最大值
     */
    @NotNull(message = "油压最大值不能为空")
    private BigDecimal oliMax;
    /**
     * 油压最小电压
     */
    @NotNull(message = "油压最小电压不能为空")
    private BigDecimal oilMinVoltage;
    /**
     * 油压最大电压
     */
    @NotNull(message = "油压最大电压不能为空")
    private BigDecimal oilMaxVoltage;
    /**
     * 输压最小值
     */
    @NotNull(message = "输压最小值不能为空")
    private BigDecimal inputPressureMinValue;
    /**
     * 输压最大值
     */
    @NotNull(message = "输压最大值不能为空")
    private BigDecimal inputPressureMaxValue;
    /**
     * 输压最小电压
     */
    @NotNull(message = "输压最小电压不能为空")
    private BigDecimal inputVoltageMinValue;
    /**
     * 输压最大电压
     */
    @NotNull(message = "输压最大电压不能为空")
    private BigDecimal inputVoltageMaxValue;
    /**
     * 流量最小值
     */
    @NotNull(message = "流量最小值不能为空")
    private BigDecimal flowRateMinValue;
    /**
     * 流量最大值
     */
    @NotNull(message = "流量最大值不能为空")
    private BigDecimal flowRateMaxValue;
    /**
     * 流量最小电压
     */
    @NotNull(message = "流量最小电压不能为空")
    private BigDecimal flowVoltageMinValue;
    /**
     * 流量最大电压
     */
    @NotNull(message = "流量最大电压不能为空")
    private BigDecimal flowVoltageMaxValue;
    /**
     * 连续采样间隔
     */
    private int continuousSamplingIntervalDuration;
    /**
     * 到达传感器有效电平
     */
    private int sensorSignalEffectiveLevel;
    /**
     * 套压补偿极性
     */
    private int pressureCompensationPolarityFlag;
    /**
     * 套压补偿值
     */
    private int pressureCompensationValueSetting;
    /**
     * 油压补偿极性
     */
    private int oilPressureCompensationPolarityFlag;
    /**
     * 油压补偿值
     */
    private int oilPressureCompensationValueSetting;
    /**
     * 输压补偿极性
     */
    private int inputPressureCompensationPolarityFlag;
    /**
     * 输压补偿值
     */
    private int inputPressureCompensationValueSetting;
    /**
     * 流量补偿极性
     */
    private int flowCompensationPolarityFlag;
    /**
     * 流量补偿值
     */
    private int flowCompensationValueSetting;

    public ControlMode() {
        this.setCode("SCSS.CONTROL_MODE");
    }

    @Override
    protected Collection<ModbusCommandDto> builderModbusCommand() {
        StringBuilder command = new StringBuilder(250);
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     0032    0036     6C
        command.append("0110003200366C");
        command.append(StringUtils.leftPad(Integer.toHexString(this.ctlModel), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.minPressure.multiply(ONE_HUNDRED).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.maxPressure.multiply(ONE_HUNDRED).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.pressureMinVoltage.multiply(ONE_HUNDRED).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.pressureMaxVoltage.multiply(ONE_HUNDRED).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.oilMin.multiply(ONE_HUNDRED).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.oliMax.multiply(ONE_HUNDRED).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.oilMinVoltage.multiply(ONE_HUNDRED).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.oilMaxVoltage.multiply(ONE_HUNDRED).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.inputPressureMinValue.multiply(ONE_HUNDRED).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.inputPressureMaxValue.multiply(ONE_HUNDRED).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.inputVoltageMinValue.multiply(ONE_HUNDRED).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.inputVoltageMaxValue.multiply(ONE_HUNDRED).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.flowRateMinValue.multiply(ONE_HUNDRED).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.flowRateMaxValue.multiply(ONE_HUNDRED).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.flowVoltageMinValue.multiply(ONE_HUNDRED).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.flowVoltageMaxValue.multiply(ONE_HUNDRED).intValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.continuousSamplingIntervalDuration), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.sensorSignalEffectiveLevel), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.pressureCompensationPolarityFlag), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.pressureCompensationValueSetting), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.oilPressureCompensationPolarityFlag), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.oilPressureCompensationValueSetting), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.inputPressureCompensationPolarityFlag), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.inputPressureCompensationValueSetting), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.flowCompensationPolarityFlag), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.flowCompensationValueSetting), 8, "0"));
        return List.of(ModbusCommandDto.builder().command(command.toString()).length(16).build());
    }
}
